@@ -16,12 +16,12 @@ CFAxisLongitude <- R6::R6Class("CFAxisLongitude",
     #' @param grp The group that contains the netCDF variable.
     #' @param nc_var The netCDF variable that describes this instance.
     #' @param nc_dim The netCDF dimension that describes the dimensionality.
-    #' @param values The dimension values of this axis.
+    #' @param values The coordinates of this axis.
     initialize = function(grp, nc_var, nc_dim, values) {
       super$initialize(grp, nc_var, nc_dim, "X", values)
     },
 
-    #' @description Return an axis spanning a smaller dimension range. This
+    #' @description Return an axis spanning a smaller coordinate range. This
     #'   method returns an axis which spans the range of indices given by the
     #'   `rng` argument.
     #'
@@ -34,18 +34,19 @@ CFAxisLongitude <- R6::R6Class("CFAxisLongitude",
     #'   [CFAxisScalar] instance is returned with the value from this axis. If
     #'   the value of the argument is `NULL`, return the entire axis (possibly
     #'   as a scalar axis).
-    sub_axis = function(group, rng = NULL) {
+    subset = function(group, rng = NULL) {
       var <- NCVariable$new(-1L, self$name, group, "NC_DOUBLE", 1L, NULL)
 
       .make_scalar <- function(idx) {
         scl <- CFAxisScalar$new(group, var, "X", idx)
         bnds <- self$bounds
         if (inherits(bnds, "CFBounds")) scl$bounds <- bnds$sub_bounds(group, idx)
+        private$subset_coordinates(scl, idx)
         scl
       }
 
       if (is.null(rng)) {
-        if (length(self$values) > 1L) {
+        if (length(private$values) > 1L) {
           ax <- self$clone()
           ax$group <- group
           ax
@@ -53,12 +54,13 @@ CFAxisLongitude <- R6::R6Class("CFAxisLongitude",
           .make_scalar(1L)
       } else {
         if (rng[1L] == rng[2L])
-          .make_scalar(self$values[rng[1L]])
+          .make_scalar(private$values[rng[1L]])
         else {
           dim <- NCDimension$new(-1L, self$name, rng[2L] - rng[1L] + 1L, FALSE)
-          lon <- CFAxisLongitude$new(group, var, dim, self$values[rng[1L]:rng[2L]])
+          lon <- CFAxisLongitude$new(group, var, dim, private$values[rng[1L]:rng[2L]])
           bnds <- self$bounds
           if (inherits(bnds, "CFBounds")) lon$bounds <- bnds$sub_bounds(group, rng)
+          private$subset_coordinates(lon, idx)
           lon
         }
       }
