@@ -47,7 +47,7 @@ open_ncdf <- function(resource, keep_open = FALSE) {
   axes <- .buildAxes(root)
 
   # Find the id's of any "bounds" variables
-  bnds <- sapply(root$NCvars, function(v) {
+  bnds <- sapply(root$NCvars, function(v) { # FIXME: what about dims in subgroups?
     nm <- v$attribute("bounds")
     if (!is.na(nm)) {
       obj <- v$group$find_by_name(nm, "NC")
@@ -75,10 +75,13 @@ open_ncdf <- function(resource, keep_open = FALSE) {
   # Auxiliary CVs and scalar CVs
   .makeCoordinates(root)
 
+  # Cell measure variables
+  .makeCellMeasures(root, axes)
+
   # Coordinate reference systems
   .makeCRS(root)
 
-  if (length(axes) > 0L) {
+  if (length(axes)) {
     # Try to identify the type of the file
     ft <- root$attribute("featureType")
     if (!is.na(ft) && ft %in% c("point", "timeSeries", "trajectory", "profile",
@@ -218,7 +221,7 @@ peek_ncdf <- function(resource) {
     if (length(dim_names)) {
       local_vars <- grp$NCvars[dim_names]
       local_CVs <- local_vars[lengths(local_vars) > 0L]
-      axes <- lapply(local_CVs, function(v) .makeAxis(grp, v, visible_dims))
+      axes <- lapply(local_CVs, function(v) .makeAxis(grp, v))
       grp$CFaxes <- append(grp$CFaxes, unlist(axes))
     } else axes <- list()
   } else axes <- list()
@@ -235,52 +238,47 @@ peek_ncdf <- function(resource) {
 # Create an `CFAxis` from an NC variable and dimension
 #
 # This method creates the various kinds of axes, with the exception of
-# [CFAxisVertical], which is passed off to `.makeAxisParametric` once a
-# parametric Z-axis is detected.
+# [CFAxisVertical] and [CFAxisTime], which are passed off to `.makeAxisParametric`
+# once a parametric Z-axis is detected, or `.makeTimeAxis()` for a possible
+# time axis.
 #
 # @param grp Group in which the NC variable is defined.
 # @param var `NCVariable` instance to create the axis from.
-# @param dims `NCDimension` instances visible to `var`.
 #
 # @return An instance of `CFAxis`.
-.makeAxis <- function(grp, var, dims) {
+.makeAxis <- function(grp, var) {
   h <- grp$handle
+  dim <- grp$find_dim_by_id(var$dimids[1L]) # FIXME: What about NC_CHAR axis?
 
   # Dimension values
   vals <- try(as.vector(RNetCDF::var.get.nc(h, var$name)), silent = TRUE)
   if (inherits(vals, "try-error"))
     # No dimension values so it's an identity axis
-    return(CFAxisDiscrete$new(grp, var, dims[[var$name]], ""))
-  if (is.numeric(vals)) vals <- round(vals, 5) # Drop spurious "precision"
+    return(CFAxisDiscrete$new(grp, var, dim, ""))
 
   # Does `var` have attributes?
   if (!nrow(var$attributes)) {
     # No attributes so nothing left to do
     if (var$vtype %in% c("NC_CHAR", "NC_STRING"))
-      return(CFAxisCharacter$new(grp, var, dims[[var$name]], "", vals))
-    else return(CFAxisNumeric$new(grp, var, dims[[var$name]], "", vals))
+      return(CFAxisCharacter$new(grp, var, dim, "", vals))
+    else return(CFAxisNumeric$new(grp, var, dim, "", vals))
   }
 
   # Z: standard_names and formula_terms for parametric vertical axis
   standard <- var$attribute("standard_name")
   if (!is.na(standard) && standard %in% Z_parametric_standard_names)
-    return(.makeParametricAxis(grp, var, dims[[var$name]], vals, standard))
+    return(.makeParametricAxis(grp, var, dim, vals, standard))
 
   # Does the axis have bounds?
-  CFbounds <- .readBounds(grp, var$attribute("bounds"), dims)
+  CFbounds <- .readBounds(grp, var$attribute("bounds"))
 
   # See if we have a "units" attribute that makes time
   units <- var$attribute("units")
   if (!is.na(units)) {
-    cal <- var$attribute("calendar")
-    if (is.na(cal)) cal <- "standard"
-    cf <- try(CFtime::CFtime(units, cal, vals), silent = TRUE)
-    if (!inherits(cf, "try-error")) {
-      if (!is.null(CFbounds))
-        CFtime::bounds(cf) <- CFbounds$values
-      timeaxis <- CFAxisTime$new(grp, var, dims[[var$name]], cf)
-      timeaxis$bounds <- CFbounds
-      return(timeaxis)
+    t <- .makeTimeAxis(grp, var, units, vals)
+    if (!inherits(t, "try-error")) {
+      t$bounds <- CFbounds$bounds
+      return(CFAxisTime$new(grp, var, dim, t))
     }
   }
 
@@ -299,10 +297,10 @@ peek_ncdf <- function(resource) {
   }
 
   axis <- if (orient == "X")
-    CFAxisLongitude$new(grp, var, dims[[var$name]], vals)
+    CFAxisLongitude$new(grp, var, dim, vals)
   else if (orient == "Y")
-    CFAxisLatitude$new(grp, var, dims[[var$name]], vals)
-  else CFAxisNumeric$new(grp, var, dims[[var$name]], orient, vals)
+    CFAxisLatitude$new(grp, var, dim, vals)
+  else CFAxisNumeric$new(grp, var, dim, orient, vals)
   axis$bounds <- CFbounds
 
   axis
@@ -322,8 +320,8 @@ peek_ncdf <- function(resource) {
   if (length(grp$NCdims) > 0L) {
     axes <- lapply(grp$NCdims, function(d) {
       if (d$id %in% add_dims) {
-        v <- NCVariable$new(-2L, d$name, grp, "NC_INTEGER", 1L, d$id)
-        axis <- CFAxisDiscrete$new(grp, v, d, "")
+        v <- NCVariable$new(-2L, d$name, grp, "NC_INT", 1L, d$id)
+        axis <- CFAxisDiscrete$new(grp, v, d, "", dim_only = TRUE)
         v$CF <- axis
         grp$NCvars <- append(grp$NCvars, v)
         grp$CFaxes <- append(grp$CFaxes, axis)
@@ -349,7 +347,7 @@ peek_ncdf <- function(resource) {
 #'
 #' @param grp The group where the axis is found.
 #' @param var The NC variable that defines the axis.
-#' @param dim The dimension associated with the axis.
+#' @param dim The NC dimension associated with the axis.
 #' @param vals The parameter values of the axis.
 #' @param param_name The "standard_name" attribute that names the specific
 #' parametric form of the axis.
@@ -379,6 +377,17 @@ peek_ncdf <- function(resource) {
   Z
 }
 
+.makeTimeAxis <- function(grp, var, units, vals) {
+  cal <- var$attribute("calendar")
+  if (is.na(cal)) cal <- "standard"
+  clim <- .readBounds(grp, var$attribute("climatology")) # Climatology must have bounds
+  try(if (is.null(clim))
+        CFtime::CFTime$new(units, cal, vals)
+      else
+        CFtime::CFClimatology$new(units, cal, vals, clim$bounds),
+      silent = TRUE)
+}
+
 #' Make CF constructs for "coordinates" references
 #'
 #' NC variables are scanned for a "coordinates" attribute (which must be a data
@@ -400,7 +409,7 @@ peek_ncdf <- function(resource) {
 #' @noRd
 .makeCoordinates <- function(grp) {
   if (length(grp$NCvars) > 0L) {
-    # Scan each unused NCVariable for the "coordinates" property and process.
+    # Scan each unused NCVariable for the "coordinates" attribute and process.
     # The NCVariable must have dimensional axes.
     for (refid in seq_along(grp$NCvars)) {
       v <- grp$NCvars[[refid]]
@@ -409,47 +418,63 @@ peek_ncdf <- function(resource) {
         coords <- strsplit(coords, " ", fixed = TRUE)[[1L]]
         varLon <- varLat <- bndsLon <- bndsLat <- NA
         for (cid in seq_along(coords)) {
+          found_one <- FALSE
           aux <- grp$find_by_name(coords[cid], "NC")
           if (!is.null(aux)) {
+            nd <- aux$ndims
             bounds <- aux$attribute("bounds")
 
-            # Create the auxiliary construct
-            nd <- aux$ndims
-            if (nd == 0L) {
-              # No dimensions so create a scalar axis in the group of aux
+            if (nd == 2L) {
+              # If the NCVariable aux has an attribute "units" with value
+              # "degrees_east" or "degrees_north" it is a longitude or latitude,
+              # respectively. Record the fact and move on.
+              units <- aux$attribute("units")
+              if (!is.na(units)) {
+                if (grepl("^degree(s?)(_?)(east|E)$", units)) {
+                  varLon <- aux
+                  bndsLon <- .readBounds(aux$group, bounds)
+                  found_one <- TRUE
+                } else if (grepl("^degree(s?)(_?)(north|N)$", units)) {
+                  varLat <- aux
+                  bndsLat <- .readBounds(aux$group, bounds)
+                  found_one <- TRUE
+                }
+              }
+            }
+
+            if (!found_one) {
               val <- try(RNetCDF::var.get.nc(grp$handle, aux$name), silent = TRUE)
-              if (inherits(val, "try-error")) val <- NULL
-              orient <- aux$attribute("axis")
-              if (is.na(orient)) orient <- ""
-              scalar <- CFAxisScalar$new(aux$group, aux, orient, val)
-              scalar$bounds <- .readBounds(aux$group, bounds)
-              aux$group$CFaxes[[aux$name]] <- scalar
-            } else if (aux$vtype %in% c("NC_CHAR", "NC_STRING")) {
-              # Label
-              val <- try(RNetCDF::var.get.nc(grp$handle, aux$name), silent = TRUE)
-              if (inherits(val, "try-error")) val <- NULL
-              dim <- grp$find_dim_by_id(aux$dimids[length(aux$dimids)]) # If there are 2 dimids, the first is a string length for a NC_CHAR type
-              aux$group$CFlabels[[aux$name]] <- CFLabel$new(aux$group, aux, dim, val)
-            } else {
-              if (!all(aux$dimids %in% vdimids) || nd > 2L)
-                warning("Unmatched `coordinates` value '", coords[cid], "' found in variable '", v$name, "'", call. = FALSE)
-              if (nd == 2L) {
-                # If the NCVariable aux has an attribute "units" with value
-                # "degrees_east" or "degrees_north" it is a longitude or latitude,
-                # respectively. Record the fact and move on.
-                units <- aux$attribute("units")
-                if (!is.na(units)) {
-                  if (grepl("^degree(s?)(_?)(east|E)$", units)) {
-                    varLon <- aux
-                    bndsLon <- .readBounds(aux$group, bounds, grp$dimensions())
-                  } else if (grepl("^degree(s?)(_?)(north|N)$", units)) {
-                    varLat <- aux
-                    bndsLat <- .readBounds(aux$group, bounds, grp$dimensions())
-                  }
+              if (inherits(val, "try-error")) {
+                warning("Could not read data for `coordinates` value '", coords[cid], "' found in variable '", v$name, "'.", call. = FALSE)
+                next
+              }
+
+              if (nd == 0L) {
+                # No dimensions so create a scalar axis in the group of aux
+                orient <- aux$attribute("axis")
+                if (is.na(orient)) orient <- ""
+                scalar <- CFAxisScalar$new(aux$group, aux, orient, val)
+                scalar$bounds <- .readBounds(aux$group, bounds)
+                aux$group$CFaxes[[aux$name]] <- scalar
+                found_one <- TRUE
+              } else if (aux$vtype %in% c("NC_CHAR", "NC_STRING")) {
+                # Label
+                dim <- grp$find_dim_by_id(aux$dimids[length(aux$dimids)]) # If there are 2 dimids, the first is a string length for a NC_CHAR type
+                aux$group$CFaux[[aux$name]] <- CFLabel$new(aux$group, aux, dim, val)
+                found_one <- TRUE
+              } else if (nd == 1L) {
+                # Auxiliary coordinate with a single dimension: make an axis out of it.
+                ax <- .makeAxis(grp, aux)
+                if (inherits(ax, "CFAxis")) {
+                  aux$group$CFaux[[aux$name]] <- ax
+                  found_one <- TRUE
                 }
               }
             }
           }
+
+          if (!found_one)
+            warning("Unmatched `coordinates` value '", coords[cid], "' found in variable '", v$name, "'.", call. = FALSE)
         }
 
         # Make a CFAuxiliaryLongLat if we have found a varLon and a varLat and
@@ -467,6 +492,62 @@ peek_ncdf <- function(resource) {
     lapply(grp$subgroups, function(g) .makeCoordinates(g))
 }
 
+#' Make CF constructs for "cell_measures" references
+#'
+#' NC variables are scanned for a "cell_measures" attribute (which must be a
+#' data variable or domain variable). The NC variable referenced is converted
+#' into a `CFCellMeasure` instance, in the group of that NC variable.
+#'
+#' The "cell_measures" may also be located in an external file. It is up to the
+#' caller to link to any such external file.
+#'
+#' @param grp The group to scan.
+#' @param axes List of available CF axes to use with the cell measure variables.
+#'
+#' @return Nothing. `CFCellMeasure` instances are created in the group where
+#'   the referenced NC variable is found. These will later be picked up when
+#'   CFvariables are created.
+#' @noRd
+.makeCellMeasures <- function(grp, axes) {
+  if (length(grp$NCvars) > 0L) {
+    # Scan each unused NCVariable for the "cell_measures" attribute and process.
+    for (refid in seq_along(grp$NCvars)) {
+      v <- grp$NCvars[[refid]]
+      if (!length(v$CF) && !is.na(meas <- v$attribute("cell_measures"))) {
+        meas <- trimws(strsplit(meas, " ", fixed = TRUE)[[1L]], whitespace = "[ \t\r\n\\:]")
+        meas <- meas[which(nzchar(meas))]
+        m <- grp$find_by_name(meas[2L], "NC")
+        if (is.null(m)) {
+          # External variable
+          root <- grp$root
+          ev <- root$attribute("external_variables")
+          if (is.na(ev) || !(meas[2L] %in% trimws(strsplit(ev, " ", fixed = TRUE)[[1L]]))) {
+            # FIXME: warning
+            warning("Unmatched `cell_measures` value '", meas[2L], "' found in variable '", v$name, "'", call. = FALSE)
+            next
+          }
+          # If it exists, move on, else create a cell measure variable
+          cmv <- grp$find_by_name(meas[2L])
+          if (inherits(cmv, "CFCellMeasure")) next
+          cm <- CFCellMeasure$new(root, meas[1L], meas[2L])
+          root$addCellMeasure(cm)
+        } else {
+          # Cell measures variable is internal. If it already exists, simply
+          # continue with the next iteration.
+          if (length(m$CF)) next
+          ax <- .buildVariableAxisList(m, axes)
+          cm <- CFCellMeasure$new(m$group, meas[1L], meas[2L], m, ax)
+          m$group$addCellMeasure(cm)
+        }
+      }
+    }
+  }
+
+  # Descend into subgroups
+  if (length(grp$subgroups))
+    lapply(grp$subgroups, function(g) .makeCellMeasures(g, axes))
+}
+
 #' Make CRS instances for "grid_mapping" references
 #'
 #' NC variables are scanned for a "grid_mapping_name" attribute. The NC variable
@@ -499,10 +580,9 @@ peek_ncdf <- function(resource) {
 # Utility function to read bounds values
 # grp - the current group being processed
 # bounds - the name of the "bounds" variable, or NULL if no bounds present
-# dims - List of visible NCDimension instances
 # axis_dims - number of axes on the coordinate variable; usually 1 but could be
 # 2 on an auxiliary coordinate variable
-.readBounds <- function(grp, bounds, dims, axis_dims = 1L) {
+.readBounds <- function(grp, bounds, axis_dims = 1L) {
   if (is.na(bounds)) NULL
   else {
     NCbounds <- grp$find_by_name(bounds, "NC")
@@ -515,10 +595,7 @@ peek_ncdf <- function(resource) {
           # FIXME: Flag non-standard item
           bnds <- bnds[, , 1L]
         }
-        dimid <- NCbounds$dimids[1L] # Bounds dimid is always the first listed
-        dim <- lapply(dims, function(d) if (d$id == dimid) d)
-        dim <- dim[lengths(dim) > 0L][[1L]] # There must be 1 only
-        CFBounds$new(NCbounds, dim, bnds)
+        CFBounds$new(NCbounds, grp$find_dim_by_id(NCbounds$dimids[1L]), bnds)
       }
     }
   }
@@ -539,18 +616,8 @@ peek_ncdf <- function(resource) {
     # Create variable for each unused NCVariable with dimensions
     vars <- lapply(grp$NCvars, function(v) {
       if (!length(v$CF) && v$ndims > 0L) {
-        xids <- lapply(axes, function(x) x$dimid)
-        ax <- vector("list", v$ndims)
-        for (x in 1:v$ndims) {
-          ndx <- which(sapply(xids, function(e) v$dimids[x] %in% e))
-          if (!length(ndx)) {
-            warning(paste0("Possible variable '", v$name, "' cannot be constructed because of unknown axis identifier ", v$dimids[x]))
-            # FIXME
-            return(NULL)
-          }
-          ax[[x]] <- axes[[ndx]]
-        }
-        names(ax) <- sapply(ax, function(x) x$name)
+        ax <- .buildVariableAxisList(v, axes)
+        ax_names <- names(ax)
         var <- CFVariable$new(grp, v, ax)
 
         # Add references to any "coordinates" of the variable
@@ -558,34 +625,52 @@ peek_ncdf <- function(resource) {
         if (!is.na(coords <- v$attribute("coordinates"))) {
           coords <- strsplit(coords, " ", fixed = TRUE)[[1L]]
           for (cid in seq_along(coords)) {
+            if (coords[cid] %in% ax_names) next
+
             aux <- grp$find_by_name(coords[cid], "CF")
             if (!is.null(aux)) {
-              clss <- class(aux)[1L]
-              if (clss == "CFAxisScalar")
+              clss <- class(aux)
+              if (clss[1L] == "CFAxisScalar")
                 var$axes[[aux$name]] <- aux
-              else if (clss == "CFLabel") {
+              else if (clss[1L] == "CFLabel") {
                 ndx <- which(sapply(ax, function(x) x$dimid == aux$dimid))
-                if (length(ndx)) ax[[ndx]]$labels <- aux
+                if (length(ndx)) ax[[ndx]]$auxiliary <- aux
+                else {  # FIXME: record warning
+                }
+              } else if ("CFAxis" %in% clss) {
+                browser()
+                ndx <- which(sapply(ax, function(x) x$dimid == aux$dimid))
+                if (length(ndx)) ax[[ndx]]$auxiliary <- aux
                 else {  # FIXME: record warning
                 }
               } else {
                 # FIXME: Record warning
               }
             } else {
-              aux <- grp$find_by_name(coords[cid], "NC")
-              if (!is.null(aux)) {
-                units <- aux$attribute("units")
+              ll <- grp$find_by_name(coords[cid], "NC")
+              if (!is.null(ll)) {
+                units <- ll$attribute("units")
                 if (!is.na(units)) {
-                  if (grepl("^degree(s?)(_?)(east|E)$", units)) varLon <- aux
-                  else if (grepl("^degree(s?)(_?)(north|N)$", units)) varLat <- aux
+                  if (grepl("^degree(s?)(_?)(east|E)$", units)) varLon <- ll
+                  else if (grepl("^degree(s?)(_?)(north|N)$", units)) varLat <- ll
                 }
               }
             }
           }
 
           if (inherits(varLon, "NCVariable") && inherits(varLat, "NCVariable")) {
-            aux <- varLon$group$find_by_name(paste(varLon$name, varLat$name, sep = "_"), "CF")
-            if (!is.null(aux)) var$gridLongLat <- aux
+            ll <- varLon$group$find_by_name(paste(varLon$name, varLat$name, sep = "_"), "CF")
+            if (!is.null(ll)) var$gridLongLat <- ll
+          }
+        }
+
+        # Add cell_measures
+        if (!is.na(cm <- v$attribute("cell_measures"))) {
+          meas <- trimws(strsplit(cm, " ", fixed = TRUE)[[1L]][2L])
+          cmv <- v$group$find_by_name(meas, "CF")
+          if (inherits(cmv, "CFCellMeasure")) {
+            var$cell_measure <- cmv
+            cmv$register(var)
           }
         }
 
@@ -612,6 +697,28 @@ peek_ncdf <- function(resource) {
   vars
 }
 
+#' Build a list of axes that a NC variable references.
+#'
+#' @param ncvar The NC variable to build the axis list for.
+#' @param axes List of available CF axes to use with the CF variables.
+#' @return List of axes for the NC variable.
+#' @noRd
+.buildVariableAxisList <- function(ncvar, axes) {
+  xids <- lapply(axes, function(x) x$dimid)
+  ax <- vector("list", ncvar$ndims)
+  for (x in 1:ncvar$ndims) {
+    ndx <- which(sapply(xids, function(e) ncvar$dimids[x] %in% e))
+    if (!length(ndx)) {
+      warning(paste0("Possible variable '", ncvar$name, "' cannot be constructed because of unknown axis identifier ", ncvar$dimids[x]))
+      # FIXME
+      return(NULL)
+    }
+    ax[[x]] <- axes[[ndx]]
+  }
+  names(ax) <- sapply(ax, function(x) x$name)
+  ax
+}
+
 # Read the attributes for a group or a variable
 .readAttributes <- function(h, name, num) {
   atts <- do.call(rbind, lapply(0L:(num - 1L), function (a) as.data.frame(RNetCDF::att.inq.nc(h, name, a))))
